@@ -1,11 +1,13 @@
 from __future__ import annotations
 
 import collections
+import copy
 import warnings
 
+import npx
 import numpy as np
 
-from ._common import _topological_dimension
+from ._common import _topological_dimension, num_nodes_per_cell
 
 
 class CellBlock(collections.namedtuple("CellBlock", ["type", "data"])):
@@ -45,7 +47,13 @@ class Mesh:
             ]
         else:
             self.cells = [
-                CellBlock(cell_type, np.asarray(data)) for cell_type, data in cells
+                CellBlock(
+                    cell_type,
+                    # polyhedron data cannot be convert to numpy arrays because the
+                    # sublists don't all have the same length
+                    data if cell_type.startswith("polyhedron") else np.asarray(data),
+                )
+                for cell_type, data in cells
             ]
         self.point_data = {} if point_data is None else point_data
         self.cell_data = {} if cell_data is None else cell_data
@@ -126,6 +134,9 @@ class Mesh:
 
         return "\n".join(lines)
 
+    def copy(self):
+        return copy.deepcopy(self)
+
     def prune(self):
         # nschloe, 2020-11:
         warnings.warn(
@@ -173,7 +184,7 @@ class Mesh:
         if flat:
             all_cells_flat = np.concatenate([c.data.flat for c in self.cells])
         else:
-            all_cells_flat = []
+            all_cells_flat = np.array([], dtype=int)
         orphaned_nodes = np.setdiff1d(np.arange(len(self.points)), all_cells_flat)
 
         if len(orphaned_nodes) == 0:
@@ -221,7 +232,7 @@ class Mesh:
 
     def get_cells_type(self, cell_type: str):
         if not any(c.type == cell_type for c in self.cells):
-            return np.array([], dtype=int)
+            return np.empty((0, num_nodes_per_cell[cell_type]), dtype=int)
         return np.concatenate([c.data for c in self.cells if c.type == cell_type])
 
     def get_cell_data(self, name: str, cell_type: str):
@@ -344,7 +355,7 @@ class Mesh:
         keys = []
         for key, data in self.cell_data.items():
             # handle all int and uint data
-            if not np.all(v.dtype.kind in ["i", "u"] for v in data):
+            if not all(v.dtype.kind in ["i", "u"] for v in data):
                 continue
 
             keys.append(key)
@@ -375,7 +386,7 @@ class Mesh:
         keys = []
         for key, data in self.point_data.items():
             # handle all int and uint data
-            if not np.all(v.dtype.kind in ["i", "u"] for v in data):
+            if not all(v.dtype.kind in ["i", "u"] for v in data):
                 continue
 
             keys.append(key)
@@ -401,3 +412,25 @@ class Mesh:
         # remove the cell data
         for key in keys:
             del self.point_data[key]
+
+    def deduplicate_points(self, tol: float) -> int:
+        num_points_before = len(self.points)
+
+        unique_points, idx, inv = npx.unique(
+            self.points, tol=tol, axis=0, return_index=True, return_inverse=True
+        )
+        self.points = unique_points
+        num_removed_points = num_points_before - len(self.points)
+
+        # re-index cell blocks
+        self.cells = [CellBlock(cb.type, inv[cb.data]) for cb in self.cells]
+
+        # reset point data
+        for key, values in self.point_data.items():
+            self.point_data[key] = values[idx]
+
+        # reset point sets
+        for key, values in self.point_sets.items():
+            self.point_sets[key] = inv[values]
+
+        return num_removed_points
