@@ -392,7 +392,7 @@ def _write_entities(fh, cells, tag_data, cell_sets, point_data, binary):
     #      numPhysicalTags(size_t) physicalTag(int) ...
     #      numBoundngSurfaces(size_t) surfaceTag(int) ...
 
-    # Both nodes and cells have enities, but the cell entities are a subset of
+    # Both nodes and cells have entities, but the cell entities are a subset of
     # the nodes. The reason is (if the inner workings of Gmsh has been correctly
     # understood) that node entities are assigned to all
     # objects necessary to specify the geometry whereas only cells of Physical
@@ -555,9 +555,7 @@ def _write_nodes(fh, points, cells, point_data, float_fmt, binary):
     if points.shape[1] == 2:
         # msh4 requires 3D points, but 2D points given.
         # Appending 0 third component.
-        points = np.column_stack(
-            [points[:, 0], points[:, 1], np.zeros(points.shape[0])]
-        )
+        points = np.column_stack([points, np.zeros_like(points[:, 0])])
 
     fh.write(b"$Nodes\n")
 
