@@ -78,9 +78,9 @@ def read_buffer(f):
         raise ReadError("Expected ply")
 
     line = _next_line(f)
+    endianness = None
     if line == "format ascii 1.0":
         is_binary = False
-        endianness = None
     elif line == "format binary_big_endian 1.0":
         is_binary = True
         endianness = ">"
@@ -110,6 +110,7 @@ def read_buffer(f):
             line = _next_line(f)
             while line[:8] == "property":
                 m = re.match("property (.+) (.+)", line)
+                assert m is not None
                 point_data_formats.append(m.groups()[0])
                 point_data_names.append(m.groups()[1])
                 line = _next_line(f)
@@ -125,9 +126,11 @@ def read_buffer(f):
             while line[:8] == "property":
                 if line[:13] == "property list":
                     m = re.match("property list (.+) (.+) (.+)", line)
+                    assert m is not None
                     cell_data_dtypes.append(tuple(m.groups()[:-1]))
                 else:
                     m = re.match("property (.+) (.+)", line)
+                    assert m is not None
                     cell_data_dtypes.append(m.groups()[0])
                 cell_data_names.append(m.groups()[-1])
                 line = _next_line(f)
@@ -218,12 +221,12 @@ def _read_ascii(
                     n = int(data[i])
                     i += n + 1
                 else:
-                    n = 1
                     cell_data[name] = collections.defaultdict(list)
                     i += 1
 
         # go over the line
         i = 0
+        n = None
         for name, dtype in zip(cell_data_names, cell_dtypes):
             if name == "vertex_indices":
                 idx_dtype, value_dtype = dtype
@@ -238,6 +241,7 @@ def _read_ascii(
             else:
                 dtype = ply_to_numpy_dtype[dtype]
                 # use n from vertex_indices
+                assert n is not None
                 cell_data[name][n] += [dtype(data[j]) for j in range(i, i + 1)]
                 i += 1
 
@@ -351,7 +355,7 @@ def _read_binary_list(buffer, count_dtype, data_dtype, num_cells, endianness):
     byte_starts_ends = np.fromiter(parse_ragged(0, num_cells), np.intp, num_cells + 1)
 
     # Next, find where the row length changes and list the (start, end) row ids
-    # of each homogenous block into `block_bounds`.
+    # of each homogeneous block into `block_bounds`.
     row_lengths = np.diff(byte_starts_ends)
     count_changed_ids = np.nonzero(np.diff(row_lengths))[0] + 1
 
@@ -362,7 +366,7 @@ def _read_binary_list(buffer, count_dtype, data_dtype, num_cells, endianness):
         start = end
     block_bounds.append((start, len(byte_starts_ends) - 1))
 
-    # Finally, parse each homogenous block. Constructing an appropriate
+    # Finally, parse each homogeneous block. Constructing an appropriate
     # `block_dtype` to include the initial counts in each row avoids any
     # wasteful copy operations.
     blocks = []
@@ -384,7 +388,7 @@ def _read_binary_list(buffer, count_dtype, data_dtype, num_cells, endianness):
     return byte_starts_ends[-1], blocks
 
 
-def write(filename, mesh, binary=True):  # noqa: C901
+def write(filename, mesh: Mesh, binary: bool = True):  # noqa: C901
 
     with open_file(filename, "wb") as fh:
         fh.write(b"ply\n")
