@@ -622,9 +622,9 @@ def write(filename, mesh, binary=True, compression="zlib", header_type=None):
             "VTU requires 3D points, but 2D points given. "
             "Appending 0 third component."
         )
-        mesh.points = np.column_stack(
-            [mesh.points[:, 0], mesh.points[:, 1], np.zeros(mesh.points.shape[0])]
-        )
+        points = np.column_stack([mesh.points, np.zeros_like(mesh.points[:, 0])])
+    else:
+        points = mesh.points
 
     vtk_file = ET.Element(
         "VTKFile",
@@ -642,7 +642,7 @@ def write(filename, mesh, binary=True, compression="zlib", header_type=None):
     assert header_type is not None
 
     if binary and compression:
-        # TODO lz4, lzma <https://vtk.org/doc/nightly/html/classvtkDataCompressor.html>
+        # TODO lz4 <https://vtk.org/doc/nightly/html/classvtkDataCompressor.html>
         compressions = {
             "lzma": "vtkLZMADataCompressor",
             "zlib": "vtkZLibDataCompressor",
@@ -653,7 +653,7 @@ def write(filename, mesh, binary=True, compression="zlib", header_type=None):
     # swap the data to match the system byteorder
     # Don't use byteswap to make sure that the dtype is changed; see
     # <https://github.com/numpy/numpy/issues/10372>.
-    points = mesh.points.astype(mesh.points.dtype.newbyteorder("="), copy=False)
+    points = points.astype(points.dtype.newbyteorder("="), copy=False)
     for k, (cell_type, data) in enumerate(mesh.cells):
         # Treatment of polyhedra is different from other types
         if is_polyhedron_grid:
@@ -741,10 +741,10 @@ def write(filename, mesh, binary=True, compression="zlib", header_type=None):
             da.text_writer = text_writer_ascii
 
     def _polyhedron_face_cells(face_cells):
-        # Define the faces of each cell on the format specfied for VTU Polyhedron cells.
-        # These are defined in Mesh.polyhedron_faces, as block data. The block consists
-        # of a nested list (outer list represents cell, inner is faces for this cells),
-        # where the items of the inner list are the nodes of specific faces.
+        # Define the faces of each cell on the format specified for VTU Polyhedron
+        # cells. These are defined in Mesh.polyhedron_faces, as block data. The block
+        # consists of a nested list (outer list represents cell, inner is faces for this
+        # cells), where the items of the inner list are the nodes of specific faces.
         #
         # The output format is specified at https://vtk.org/Wiki/VTK/Polyhedron_Support
 
