@@ -1,7 +1,7 @@
+from __future__ import annotations
+
 import copy
 import string
-import tempfile
-from pathlib import Path
 
 import numpy as np
 
@@ -593,22 +593,25 @@ lagrange_high_order_mesh = meshio.Mesh(
 
 
 def add_point_data(mesh, dim, num_tags=2, seed=0, dtype=float):
-    np.random.seed(seed)
+    rng = np.random.default_rng(seed)
+
     mesh2 = copy.deepcopy(mesh)
 
     shape = (len(mesh.points),) if dim == 1 else (len(mesh.points), dim)
-    data = [(100 * np.random.rand(*shape)).astype(dtype) for _ in range(num_tags)]
+    data = [(100 * rng.random(shape)).astype(dtype) for _ in range(num_tags)]
 
     mesh2.point_data = {string.ascii_lowercase[k]: d for k, d in enumerate(data)}
     return mesh2
 
 
-def add_cell_data(mesh, specs):
+def add_cell_data(mesh, specs: list[tuple[str, tuple[int, ...], type]]):
     mesh2 = copy.deepcopy(mesh)
-    np.random.seed(0)
+
+    rng = np.random.default_rng(0)
+
     mesh2.cell_data = {
         name: [
-            (100 * np.random.rand(*((len(cells),) + shape))).astype(dtype)
+            (100 * rng.random((len(cells),) + shape)).astype(dtype)
             for _, cells in mesh.cells
         ]
         for name, shape, dtype in specs
@@ -647,14 +650,13 @@ def add_cell_sets(mesh):
     return mesh2
 
 
-def write_read(writer, reader, input_mesh, atol, extension=".dat"):
+def write_read(tmp_path, writer, reader, input_mesh, atol, extension=".dat"):
     """Write and read a file, and make sure the data is the same as before."""
     in_mesh = copy.deepcopy(input_mesh)
 
-    with tempfile.TemporaryDirectory() as temp_dir:
-        p = Path(temp_dir) / ("test" + extension)
-        writer(p, input_mesh)
-        mesh = reader(p)
+    p = tmp_path / ("test" + extension)
+    writer(p, input_mesh)
+    mesh = reader(p)
 
     # Make sure the output is writeable
     assert mesh.points.flags["WRITEABLE"]
@@ -745,12 +747,10 @@ def write_read(writer, reader, input_mesh, atol, extension=".dat"):
             assert np.allclose(var1, var2, atol=atol, rtol=0.0)
 
 
-def generic_io(filename):
-    with tempfile.TemporaryDirectory() as temp_dir:
-        filepath = Path(temp_dir) / filename
-        meshio.write_points_cells(filepath, tri_mesh.points, tri_mesh.cells)
-        out_mesh = meshio.read(filepath)
-        assert (abs(out_mesh.points - tri_mesh.points) < 1.0e-15).all()
-        for c0, c1 in zip(tri_mesh.cells, out_mesh.cells):
-            assert c0.type == c1.type
-            assert (c0.data == c1.data).all()
+def generic_io(filepath):
+    meshio.write_points_cells(filepath, tri_mesh.points, tri_mesh.cells)
+    out_mesh = meshio.read(filepath)
+    assert (abs(out_mesh.points - tri_mesh.points) < 1.0e-15).all()
+    for c0, c1 in zip(tri_mesh.cells, out_mesh.cells):
+        assert c0.type == c1.type
+        assert (c0.data == c1.data).all()
